package de.szut.lf8_starter.employee;

import de.szut.lf8_starter.exceptionHandling.ResourceNotFoundException;
import de.szut.lf8_starter.project.ProjectEntity;
import de.szut.lf8_starter.project.ProjectRepository;
import de.szut.lf8_starter.project.dto.employee.GetEmployeeDto;
import de.szut.lf8_starter.project.dto.project.GetProjectDto;
import de.szut.lf8_starter.project.mapper.ProjectMapper;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Service;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.List;

@Service
public class EmployeeService {
    private final RestTemplate restTemplate;
    private final ProjectRepository projectRepository;
    private final ProjectMapper projectMapper;

    public EmployeeService(RestTemplate restTemplate, ProjectRepository projectRepository, ProjectMapper projectMapper) {
        this.restTemplate = restTemplate;
        this.projectRepository = projectRepository;
        this.projectMapper = projectMapper;
    }

    public boolean employeeExists(String accessToken, Long employeeId) {
        HttpHeaders headers = new HttpHeaders();
        headers.setBearerAuth(accessToken.replace("Bearer ", ""));

        HttpEntity<String> requestEntity = new HttpEntity<>(headers);

        String url = "https://employee.szut.dev/employees/" + employeeId;

        try {
            restTemplate.exchange(url, HttpMethod.GET, requestEntity, String.class);
        } catch (HttpClientErrorException.NotFound e) {
            return false;
        }

        return true;
    }

    public GetEmployeeDto getEmployee(String accessToken, Long employeeId) {
        HttpHeaders headers = new HttpHeaders();
        headers.setBearerAuth(accessToken.replace("Bearer ", ""));

        HttpEntity<String> requestEntity = new HttpEntity<>(headers);

        String url = "https://employee.szut.dev/employees/" + employeeId;

        try {
            return restTemplate.exchange(url, HttpMethod.GET, requestEntity, GetEmployeeDto.class).getBody();
        } catch (HttpClientErrorException.NotFound e) {
            throw new ResourceNotFoundException("Employee not found");
        }
    }

    public List<GetProjectDto> getProjects(Long employeeId) {
        ArrayList<ProjectEntity> projects = this.projectRepository.findAllByEmployeeId(employeeId);
        ArrayList<GetProjectDto> projectDtos = new ArrayList<>();

        for (ProjectEntity project : projects) {
            projectDtos.add(this.projectMapper.mapToGetDto(project));
        }

        return projectDtos;
    }
}
