package de.szut.lf8_starter.project;

import de.szut.lf8_starter.project.dto.project.GetProjectDto;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
public class ProjectService {
    private final ProjectRepository projectRepository;

    public ProjectService(ProjectRepository projectRepository) {
        this.projectRepository = projectRepository;
    }

    public ProjectEntity create(ProjectEntity projectEntity) {
        return this.projectRepository.save(projectEntity);
    }

    public List<ProjectEntity> readAll() {
        return this.projectRepository.findAll();
    }

    public Optional<ProjectEntity> findById(Long id) {
        return projectRepository.findById(id);
    }

    public void update(ProjectEntity project) {
        this.projectRepository.save(project);
    }

    public void delete(Long id) {
        this.projectRepository.deleteById(id);
    }

    public boolean isOverlapping(GetProjectDto getProjectDto, ProjectEntity existingProjectEntity) {
        return isDateRangeOverlapping(getProjectDto, existingProjectEntity) || isDateEqual(getProjectDto, existingProjectEntity);
    }

    private boolean isDateRangeOverlapping(GetProjectDto getProjectDto, ProjectEntity existingProjectEntity) {
        return getProjectDto.getStartDate().isBefore(existingProjectEntity.getPlannedEndDate())
                && getProjectDto.getPlannedEndDate().isAfter(existingProjectEntity.getStartDate());
    }

    private boolean isDateEqual(GetProjectDto getProjectDto, ProjectEntity existingProjectEntity) {
        return getProjectDto.getStartDate().isEqual(existingProjectEntity.getStartDate())
                || getProjectDto.getPlannedEndDate().isEqual(existingProjectEntity.getPlannedEndDate());
    }
}
