package de.szut.lf8_starter.project.action.employee;

import de.szut.lf8_starter.employee.EmployeeService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RestController;

@RestController
public class GetProjectsFromEmployee {
    private final EmployeeService employeeService;

    public GetProjectsFromEmployee(EmployeeService employeeService) {
        this.employeeService = employeeService;
    }

    @Operation(summary = "Get all projects from employee")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Get all projects from employee", content = @Content),
            @ApiResponse(responseCode = "404", description = "Employee not found", content = @Content)
    })
    @GetMapping("/employees/{employeeId}/projects")
    public ResponseEntity<Object> create(
            @PathVariable Long employeeId,
            @RequestHeader("Authorization") String accessToken
    ) {
        if (!this.employeeService.employeeExists(accessToken, employeeId)) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }

        return new ResponseEntity<>(this.employeeService.getProjects(employeeId), HttpStatus.OK);
    }
}
