package de.szut.lf8_starter.integration.project;

import de.szut.lf8_starter.project.ProjectEntity;
import de.szut.lf8_starter.project.ProjectRepository;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

import java.time.LocalDate;
import java.util.List;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc(addFilters = false)
@DirtiesContext(classMode = DirtiesContext.ClassMode.AFTER_EACH_TEST_METHOD)
class RemoveProjectActionTest {
    @Autowired
    private MockMvc mockMvc;
    @Autowired
    private ProjectRepository projectRepository;

    @Test
    void deleteProjectTest() throws Exception {
        ProjectEntity project = new ProjectEntity();
        project.setComment("comment");
        project.setContractor(1);
        project.setContractorName("contractorName");
        project.setEndDate(LocalDate.of(2024, 1, 1));
        project.setLeadingEmployee(1);
        project.setName("name");
        project.setStartDate(LocalDate.of(2021, 1, 1));
        project.setEmployees(List.of(1L, 2L, 3L));
        this.projectRepository.save(project);

        this.mockMvc.perform(delete("/projects/" + project.getId()))
                .andExpect(status().isNoContent());
    }

    @Test
    void deleteProjectShouldReturnNotFoundResponseWhenProjectIsNotFound() throws Exception {
        this.mockMvc.perform(get("/projects/2987")).andExpect(status().isNotFound());
    }
}
